@@ -26,6 +26,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef __FreeBSD__
+#include <bsd/string.h> // strlcpy
+#endif
 #include <syslog.h>
 #include <time.h>
 #include <unistd.h>
@@ -76,12 +79,7 @@
 #include <sys/sysctl.h>
 #endif
 
-#if !defined(__GLIBC__) && defined(linux)
-#include <linux/if.h>
-#define IF_NAMESIZE IFNAMSIZ
-#else
 #include <net/if.h>
-#endif
 
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
