@@ -16,8 +16,7 @@ size_t snprint_safe_buffer(char *s, size_t size, struct safe_buffer const *sb)
 {
 	size_t count = 0;
 
-	count += snprintf((s + count), (size - count), "unsigned char expected[] = {");
-	count--;
+	count += snprintf((s + count), (size - count), "unsigned char expected[] = { /* sb.allocated = %ld, sb.used = %ld */", sb->allocated, sb->used);
 
 	for (size_t i = 0; i < sb->used; ++i) {
 		if (i % 8 == 0) {
@@ -25,8 +24,9 @@ size_t snprint_safe_buffer(char *s, size_t size, struct safe_buffer const *sb)
 		} else {
 			count += snprintf((s + count), (size - count), " 0x%02x,", sb->buffer[i]);
 		}
-		count--;
 	}
+	/* Do not remove the final byte's comma. Only JSON requires the comma is
+	 * removed, and this is not JSON. */
 
 	count += snprintf((s + count), (size - count), "\n};\n");
 	return count;
